@@ -98,8 +98,8 @@ struct Convert {
     #[clap(long, arg_enum, value_name = "on/off", default_value = "on")]
     indent: Boolish,
 
-    #[clap(long, default_value="*")]
-    select: preserves_path::Node,
+    #[clap(long="select", default_value="*")]
+    select_expr: String,
 
     #[clap(long, arg_enum, default_value="sequence")]
     select_output: SelectOutput,
@@ -112,6 +112,12 @@ struct Convert {
 
     #[clap(long, arg_enum, value_name = "on/off", default_value = "on")]
     write_annotations: Boolish,
+
+    #[clap(long, value_name = "filename")]
+    bundle: Vec<std::path::PathBuf>,
+
+    #[clap(long)]
+    schema: Option<String>,
 }
 
 #[derive(ArgEnum, Clone, Debug)]
@@ -133,7 +139,6 @@ struct StringQuotation {
     escape_spaces: bool,
 }
 
-
 #[derive(Clap, Clone, Debug)]
 enum QuotationOutput {
     String(StringQuotation),
@@ -152,11 +157,11 @@ struct Quote {
 
 #[derive(Clap, Clone, Debug)]
 enum Subcommand {
-    Convert(Convert),
     Completions {
         #[clap(arg_enum, value_name = "dialect")]
         dialect: CompletionDialect,
     },
+    Convert(Convert),
     Quote(Quote),
 }
 
@@ -169,7 +174,7 @@ struct CommandLine {
 
 fn main() -> io::Result<()> {
     let args = CommandLine::parse();
-    match args.command {
+    Ok(match args.command {
         Subcommand::Completions { dialect } => {
             let mut app = CommandLine::into_app();
             match dialect {
@@ -182,8 +187,7 @@ fn main() -> io::Result<()> {
         },
         Subcommand::Convert(c) => convert(c)?,
         Subcommand::Quote(q) => quote(q)?,
-    }
-    Ok(())
+    })
 }
 
 struct RollingBuffer<R: io::Read> {
@@ -365,6 +369,14 @@ fn print_unquoted(v: &IOValue) {
 }
 
 fn convert(c: Convert) -> io::Result<()> {
+    let mut env = preserves_path::Env::new();
+    for f in c.bundle.iter() {
+        env.load_bundle(f)?;
+    }
+    let select = preserves_path::Node::from_str(&env, &c.select_expr)
+        .map_err(|e| io::Error::new(
+            io::ErrorKind::InvalidData,
+            format!("Invalid select expression: {}: {:?}", e, c.select_expr)))?;
     let mut vs = ValueStream::new(c.input_format, c.read_annotations.into(), io::stdin());
     let write_ann: bool = c.write_annotations.into();
     let mut w: Box<dyn FnMut(&IOValue) -> io::Result<()>> = match c.output_format {
@@ -385,18 +397,24 @@ fn convert(c: Convert) -> io::Result<()> {
         }
         OutputFormat::Binary => {
             let mut p = PackedWriter::new(io::stdout());
-            Box::new(move |v| if write_ann {
-                p.write(&mut IOValueDomainCodec, v)
-            } else {
-                p.write(&mut IOValueDomainCodec, &v.strip_annotations::<IOValue>())
+            Box::new(move |v| {
+                if write_ann {
+                    p.write(&mut IOValueDomainCodec, v)?;
+                } else {
+                    p.write(&mut IOValueDomainCodec, &v.strip_annotations::<IOValue>())?;
+                }
+                Ok(())
             })
         }
         OutputFormat::Unquoted =>
-            Box::new(|v| Ok(print_unquoted(v))),
+            Box::new(|v| {
+                print_unquoted(v);
+                Ok(())
+            }),
     };
     while let Some(value) = vs.next() {
         let value = value?;
-        let matches = c.select.exec(&value);
+        let matches = select.exec(&mut env.to_context(), &value);
         if c.collect {
             match c.select_output {
                 SelectOutput::Sequence => w(&IOValue::new(matches))?,
@@ -410,7 +428,7 @@ fn convert(c: Convert) -> io::Result<()> {
         }
         if let Some(limit) = c.limit {
             if vs.count >= limit {
-                return Ok(());
+                break;
             }
         }
     }
